//////////////////////////////////////////////////////////////////////
//
//  MDIDEMO.PRG
//
//  Copyright:
//      Alaska Software, (c) 1997-2019. Alle Rechte vorbehalten.         
//  
//  Inhalt:
//      Main Routine und AppSys() fr eine MDI Beispielanwendung
//   
//////////////////////////////////////////////////////////////////////


#include "Gra.ch"
#include "Xbp.ch"
#include "AppEvent.ch"
#include "Mdidemo.ch"


/*
 * Fgen Sie die Define-Konstante USE_POSTGRES zu Ihren Projekt-Einstellungen -> Compiler
 * -> Define-Konstanten hinzu, um den PostgreSQL-Server anstatt der dbf-Tabellen und
 * -Indexe (dbf/cdx) zu verwenden.
 */
#ifdef USE_POSTGRES
#include "pgdbe.ch"
#endif


/*
 * Main Prozedur und Event-Loop
 */
PROCEDURE Main
   LOCAL nEvent, mp1, mp2, oXbp
   FIELD CUSTNO, LASTNAME, FIRSTNAME, PARTNO, PARTNAME

   /* Laden der PostgreSQL DatabaseEngine und Verbindungsaufbau mit dem Server
    * HINWEIS:
    * Bitte ndern Sie server, uid und pwd entsprechend Ihre Server installation.
    */
#ifdef USE_POSTGRES
   LOCAL oSession
   LOCAL cConnStr

   DbeLoad("pgdbe")
   DbeSetDefault("pgdbe")

   cConnStr := "DBE=pgdbe;server=localhost;"
   cConnStr += "db=mdidemo;uid=postgres;pwd=postgres"

   oSession := DacSession():New(cConnStr)
   IF(!oSession:IsConnected())
     MsgBox("Verbindung fehlgeschlagen ("+Var2Char(oSession:GetLastMessage())+")")
     QUIT
   ENDIF
#endif

   /* Die Funktion File() prft auf Dateiebene und kann deshalb nicht verwendet
    * werden, um zu prfen, ob eine Tabelle auf dem PostgreSQL-Server vorhanden
    * ist. Verwenden Sie die Funktion Table() stattdessen. Table()
    * funktioniert mit dbf-Tabellen, sql-Tabellen sowohl bei lokalen als auch
    * bei remote Daten.
    */
#ifdef USE_POSTGRES
   MsgBox("Verbindung mit PostgreSQL-Server erfolgreich")
   IF(!Table("Customer"))
     MsgBox("Keine Kunden Tabelle vorhanden")
   ENDIF
   IF(!Table("Parts"))
     MsgBox("Keine Artikel Tabelle vorhanden")
   ENDIF
#else
   // Indexdateien prfen und ggf. erzeugen
   SET DEFAULT TO ..\..\data\misc

   IF ! AllFilesExist( { "CUSTA.NTX", "CUSTB.NTX" } )
      USE Customer EXCLUSIVE
      INDEX ON CustNo                    TO CustA
      INDEX ON Upper(LastName+Firstname) TO CustB
      CLOSE DATABASE
   ENDIF

   IF ! AllFilesExist( { "PARTS.CDX" } )
      USE Parts EXCLUSIVE VIA FOXCDX
      INDEX ON Upper(PartNo)   TAG PartNo   TO Parts
      INDEX ON Upper(PartName) TAG PartName TO Parts
      CLOSE DATABASE
   ENDIF
#endif

   SET DELETED ON

   // Das Programm wird in AppQuit() abgebrochen
   DO WHILE .T.
      nEvent := AppEvent( @mp1, @mp2, @oXbp )
      oXbp:handleEvent( nEvent, mp1, mp2 )
   ENDDO

RETURN



/*
 * Feststellen, ob alle Dateien im Array 'aFiles' vorhanden sind
 */
FUNCTION AllFilesExist( aFiles )
   LOCAL lExist := .T., i:=0, imax := Len(aFiles), cPath := Set( _SET_DEFAULT )

   IF(!Empty(cPath) .AND. cPath[-1]!='\')
     cPath += "\"
   ENDIF

   DO WHILE ++i <= imax .AND. lExist
      lExist := File( cPath + aFiles[i] )
   ENDDO
RETURN lExist



/*
 * Routine zum Beenden eines Programms
 */
PROCEDURE AppQuit()
   LOCAL nButton

   nButton := ConfirmBox( , ;
                 "Wollen Sie das Programm abbrechen ?", ;
                 "Beenden", ;
                  XBPMB_YESNO , ;
                  XBPMB_QUESTION+XBPMB_APPMODAL+XBPMB_MOVEABLE )

   IF nButton == XBPMB_RET_YES
      COMMIT
      CLOSE ALL
#ifdef USE_POSTGRES
      // Immer die Verbindung(en) zum Server beenden.
      AEval( DacSession():sessionList(), {|o|o:disconnect()})
#endif

      QUIT
   ENDIF

RETURN



/*
 * Beispiel fr AppSys() in MDI Anwendung
 * Die Funktion wird vor der Main Prozedur ausgefhrt
 */
PROCEDURE AppSys
   LOCAL oDlg, oXbp, aPos[2], aSize

   // Gre vom DeskTop feststellen
   // und Anwendungsfenster dimensionieren
   aSize    := SetAppWindow():currentSize()
   aPos[1]  := 0.1 * aSize[1]
   aPos[2]  := 0.1 * aSize[2]
   aSize[1] *= 0.8
   aSize[2] *= 0.8

   // Anwendungsfenster erzeugen (application window)
   oDlg       := XbpDialog():new( ,, aPos, aSize,, .F. )
   oDlg:title := "Toys & Fun Inc. [Xbase++ - MDI Demo]"
   oDlg:icon  := ICON_APPLICATION
   oDlg:close := {|| AppQuit() }
   oDlg:taskList := .T.
   oDlg:drawingArea:ClipChildren := .T.
   oDlg:create()
   oDlg:drawingArea:SetColorBG( XBPSYSCLR_APPWORKSPACE )
   SetAppWindow( oDlg )

   // Mensystem erzeugen (UDF). Das Hauptmen ist systemweit
   // abrufbar ber GetApplication():MainForm:MenuBar()
   MenuCreate( oDlg:menuBar() )

   // Online-Hilfe mit UDF bereitstellen
   oXbp := XbpHelpLabel():new():create()
   oXbp:helpObject := ;
        HelpObject( "MDIDEMOG.HLP", "Hilfe fr MDI Demo" )
   oDlg:helpLink := oXbp

   // Anwendungsfenster anzeigen und Fokus geben
   oDlg:show()
   SetAppFocus( oDlg )

   // Anwendungsfenster ist systemweit abrufbar ber
   // GetApplication():MainForm
RETURN



/*
 * Routine fr das Help-Object. Es verwaltet die Online-Hilfe
 */
FUNCTION HelpObject( cHelpFile, cTitle )
   STATIC soHelp

   IF soHelp == NIL
      soHelp := XbpHelp():new()
      soHelp:resGeneralHelp := ID_HELP_GENERAL
      soHelp:resKeysHelp    := ID_HELP_KEYS
      soHelp:create( SetAppWindow(), cHelpFile, cTitle )
   ENDIF
RETURN soHelp
