****************************************************************
*   mybr.prg (Modul von SIMAWI)
*   Funktionen des Browse-Objektes
*
*   SINUS Messtechnik GmbH, K. Gedicke
*   Stand: 20.8.99
****************************************************************


#include "sima.ch"

#define TURN_ON_APPEND_MODE(b)      (b:cargo := .T.)
#define TURN_OFF_APPEND_MODE(b)     (b:cargo := .F.)
#define IS_APPEND_MODE(b)           (b:cargo)

#define MY_HEADSEP      CHR(196) + CHR(194) + CHR(196)
#define MY_COLSEP       " " + CHR(179) + " "
#define MY_FOOTSEP      CHR(196) + CHR(193) + CHR(196)


********************************************************
FUNC SiBrowse()
********************************************************

LOCAL nCursSave:=SetCursor(SC_NONE),;
      n:=0, column, z, nrow:=0, ncol:=0, nwidth:=0

PARAMETERS nTop, nLeft, nBottom, nRight, fu, edit

PRIVATE browse, nKey:=0, nDelayKey:=0, lMore:= .T.,;
        owa:=SELECT()

n := LASTREC()

edit:=IF(edit=NIL, .f., edit)

ntop:=IF(ntop=NIL, 0, ntop)
nleft:=IF(nleft=NIL, 0, nleft)
nbottom:=IF(nbottom=NIL, MAXROW(), nbottom)
nright:=IF(nright=NIL, MAXCOL(), nright)

browse:=TBrowseNew(ntop,nleft,nbottom,nright)
browse:skipBlock     := { |x| Skipper(x, browse) }
browse:goTopBlock    := { || DBGOTOP() }
browse:goBottomBlock := { || DBGOBOTTOM() }
browse:headSep       := MY_HEADSEP
browse:colSep        := MY_COLSEP
#ifdef __XPP__
browse:colorSpec     := farbe1
#endif


IF !EMPTY(BrowseBl)
    EVAL(BrowseBl)
ELSE
    FOR n:=1 TO FCOUNT()
        column:=TBColumnNew( FieldName(n), FieldBlock( FieldName(n) ) )
        browse:addColumn(column)
    NEXT
    AddRecno(browse)
END
browse:configure()  // ntig damit browse:colWidth(n) funktioniert
nwidth := nright + 1 - nleft
FOR n:=1 TO browse:colCount
    nwidth -= (browse:colWidth(n) + 3)  // +3 fr Spaltentrenner " | "
NEXT
IF nwidth > 3
    column := TBColumnNew(SPACE(nwidth - 3), {|| ""})
    browse:addColumn(column)
END

IF edit .AND. EOF()
    TURN_ON_APPEND_MODE(browse)
ELSE
    TURN_OFF_APPEND_MODE(browse)
END

#ifdef __XPP__
DISPBEGIN()
#endif
WHILE lMore
    IF browse:colPos <= browse:freeze
        browse:colPos := browse:freeze + 1
    END

    nKey:=0
    WHILE nKey==0 .AND. !browse:stabilize()
        nKey:=INKEY()
    END

    IF browse:stable
        nrow:=ROW()     //Cursorposition merken
        ncol:=COL()
        IF !EMPTY(fu)
            EVAL(fu)    //zustzlichen (Anzeige-) Block ausfhren
        END
        SETPOS(nrow, ncol)  //Cursorposition wieder herstellen

        IF browse:hitBottom .AND. .NOT. IS_APPEND_MODE(browse) .AND. edit
            TURN_ON_APPEND_MODE(browse)
            nKey := K_DOWN
        ELSE
            IF browse:hitTop .OR. browse:hitBottom
                TONE(880,1)
            END
#ifndef __XPP__
            WHILE (nKey:=INKEY(1)) == 0
               DISPBEGIN()
#endif
                browse:refreshAll()     //Gltigkeit aufheben
                browse:forceStable()    //fr Novell-Netz
               DISPEND()
#ifdef __XPP__
            DISPBEGIN()
            IF nDelayKey == 0
                nKey := INKEY(0)
            ELSE
                nKey := nDelayKey
                nDelayKey := 0
            END
#else
            END   // WHILE (nKey:=INKEY(1)) == 0
#endif
        END   // IF browse:hitBottom...
    END   // IF browse:stable
    EApplyKey(browse, nKey)     //Tastatureingabe auswerten
END   // WHILE lMore
#ifdef __XPP__
DISPEND()
#endif
SETCURSOR(nCursSave)
RETURN (nKey==13)   // bei Taste "ENTER" .t.


******************************************************
*     EApplyKey  -  Reaktion auf Keyboardeingabe     *
******************************************************
PROC EApplyKey(browse, nKey)

DO CASE
    CASE nKey == K_DOWN
        browse:down()
    CASE nKey == K_PGDN
        browse:pageDown()
    CASE nKey == K_CTRL_PGDN
        browse:goBottom()
        TURN_OFF_APPEND_MODE(browse)
    CASE nKey == K_UP
        browse:up()
        IF IS_APPEND_MODE(browse)
            TURN_OFF_APPEND_MODE(browse)
            browse:refreshAll()
        ENDIF
    CASE nKey == K_PGUP
        browse:pageUp()
        IF IS_APPEND_MODE(browse)
            TURN_OFF_APPEND_MODE(browse)
            browse:refreshAll()
        ENDIF
    CASE nKey == K_CTRL_PGUP
        browse:goTop()
        TURN_OFF_APPEND_MODE(browse)
    CASE nKey == K_RIGHT
        browse:right()
    CASE nKey == K_LEFT
        browse:left()
    CASE nKey == K_HOME
        browse:home()
    CASE nKey == K_END
        browse:end()
    CASE nKey == K_CTRL_LEFT
        browse:panLeft()
    CASE nKey == K_CTRL_RIGHT
        browse:panRight()
    CASE nKey == K_CTRL_HOME
        browse:panHome()
    CASE nKey == K_CTRL_END
        browse:panEnd()
    CASE !EMPTY(FuncBl)
        browse:forceStable()
        EVAL(FuncBl)
        IF lMore
            browse:refreshAll()
        END
    CASE nKey == K_ESC
        lMore := .F.
    CASE nKey == K_RETURN .AND. edit
        DoGet(browse)
    OTHERWISE
        IF edit
            KEYBOARDX(CHR(nKey))
            DoGet(browse)
        ENDIF
    ENDCASE
RETURN


***************************************************************
*     KEYBOARDX  -  Inhalt des Tastaturpuffers erhalten       *
*                   z.B. Einfgen (rechte Maustaste) wenn     *
*                   Browser-Feld noch nicht im Editmodus      *
***************************************************************
PROC KEYBOARDX(cKey)
LOCAL nKeyX:=0

#ifdef __XPP__
WHILE (nKeyX := INKEY(0.1)) <> 0
    cKey += CHR(nKeyX)
END
#endif
KEYBOARD cKey
RETURN


*************************************************
*     DoGet  -  Editieren im EBrowse-Objekt     *
*************************************************
PROCEDURE DoGet(browse)

LOCAL bIns, lScore, lExit
LOCAL col, get, nKey, n:=0, nDC:=0
LOCAL lAppend, xOldKey, xNewKey

browse:forceStable()
lAppend := IS_APPEND_MODE(browse)
IF lAppend .AND. RECNO() == LASTREC() + 1
    IF !myAppend()
        RETURN
    END
END

xOldKey := IF( EMPTY(INDEXKEY()), NIL, &(INDEXKEY()) )

lScore := Set(_SET_SCOREBOARD, .F.)
lExit := Set(_SET_EXIT, .T.)
bIns := SetKey(K_INS)

SetKey( K_INS, {|| InsToggle()} )

SetCursor( IF(ReadInsert(), SC_INSERT, SC_NORMAL) )

col := browse:getColumn(browse:colPos)
get := GetNew(Row(), Col(), col:block, col:heading, col:picture, browse:colorSpec)
#ifndef __XPP__
get:PreBlock  := col:PreBlock
get:PostBlock := col:PostBlock
#endif
IF myRLock()
    nDC := DISPCOUNT()
    FOR n := 1 TO nDC   // (LIN 25.09.2009, LUD 28.09.2009) Eingabe unsichtbar
        DISPEND()       // fr "Edit in Liste", z.B. Bestellung/F2->Wareneingang
    NEXT
    ReadModal( {get} )
    FOR n := 1 TO nDC
        DISPBEGIN()
    NEXT
    DBUNLOCK()
END

// Restore state
SetCursor(0)
Set(_SET_SCOREBOARD, lScore)
Set(_SET_EXIT, lExit)
SetKey(K_INS, bIns)

// Get the record's key value (or NIL) after the GET
xNewKey := IF( EMPTY(INDEXKEY()), NIL, &(INDEXKEY()) )

// If the key has changed (or if this is a new record)
IF .NOT. (xNewKey == xOldKey) .OR. (lAppend .AND. xNewKey != NIL)

    browse:refreshAll()
    browse:forceStable()

    // Make sure we're still on the right record after stabilizing
    WHILE &(INDEXKEY()) > xNewKey .AND. .NOT. browse:hitTop
        browse:up()
        browse:forceStable()
    END
END

// For this demo, we turn append mode off after each new record
TURN_OFF_APPEND_MODE(browse)

// Check exit key from get
nKey := LASTKEY()
IF nKey == K_UP .OR. nKey == K_DOWN .OR. ;
 nKey == K_PGUP .OR. nKey == K_PGDN
#ifdef __XPP__
    nDelayKey := nKey   // sonst "Zeilensprung" durch stabilize()
#else
    KEYBOARD( CHR(nKey) )
#endif
END
RETURN


*********************************************************************
*     AddRecno  -  fgt Spalte fr Satznummer in EBrowse-Objekt     *
*********************************************************************
PROC AddRecno(browse)
LOCAL column
column := TBColumnNew( "  NR", {|| STR(RECNO(),4)} )
browse:insColumn(1, column)
browse:freeze := 1
RETURN


*********************************************************************
*     InsToggle  -  umschalten Insertmodus und Cursordarstellung    *
*********************************************************************
PROCEDURE InsToggle()
IF READINSERT()
      READINSERT(.F.)
      SETCURSOR(SC_NORMAL)
ELSE
      READINSERT(.T.)
      SETCURSOR(SC_INSERT)
ENDIF
RETURN

***************************************************
FUNC Skipper(n, browse)
***************************************************
LOCAL lAppend, i
//, nRec:=RECNO()

lAppend := IS_APPEND_MODE(browse)
i := 0
IF n == 0 .OR. LASTREC() == 0
    SKIP 0      // aktualisiert Wert fr die Anzeige!
ELSEIF n > 0 .AND. RECNO() != LASTREC() + 1
//    SKIP n
//    IF ( EOF() )
//        DBGOTO(nRec)
        DO WHILE i < n
            SKIP 1
            IF ( EOF() )
                IF ( lAppend )
                    i++
                ELSE
                    SKIP -1
                ENDIF
                EXIT
            ENDIF
            i++
        ENDDO
//    ELSE
//        i := n
//    ENDIF
ELSEIF n < 0
//    SKIP n
//    IF ( BOF() )
//        DBGOTO(nRec)
        DO WHILE i > n
            SKIP -1
            IF ( BOF() )
                EXIT
            ENDIF
            i--
        ENDDO
//    ELSE
//        i := n
//    ENDIF
ENDIF
RETURN i



**********************
FUNC myAppend(lUnLock)
//Satz anfgen mit Fehlerabfrage
LOCAL lRet:=.f.
IF lUnlock==NIL
    lUnLock:=.t.
END
#ifdef __XPP__
DBAPPEND(IIF(lUnLock, 0, 1))
#else
DBAPPEND(lUnLock)
#endif
IF lRet:=NETERR()
    ALERT(ne2 + ":;" + ALIAS())
END
RETURN !lRet


**************
FUNC myRLock()
// Satz sperren mit Warteschleife
LOCAL ret:=.f., i:=10, wi:={}
WHILE !(ret:=RLOCK())
    IF EMPTY(wi)
        wi:=swopen(10,25,14,54,,"W+/R")
        ? PADC("Bitte warten!",26)
    END
    IF i>0
        i--
//        INKEY(0.2) // warte 0,2 Sekunden
    ELSE
        ALERT("Satz sperren z.Z. unmglich! (" + ALIAS() + ")")
        EXIT
    END
END
IF !EMPTY(wi)
    swclose(wi)
END
RETURN ret



****************************************************************
// Funktionen fr Fenster

***************************
FUNC swopen(a,b,c,d,txt,co)
//Fenster ffnen
LOCAL aFeld:={}, nSC:=SETCURSOR()
AADD(aFeld, WSELECT())
AADD(aFeld, WOPEN(a,b,c,d))
#ifdef __XPP__
AADD(aFeld, nSC)
if co==NIL
    co := farbe1
end
#endif
SETCOLOR(co)
WBOX()
#ifdef __XPP__
WFORMAT(-1,0,-1,0)
IF txt!=NIL
    WSetTitle(0, 2, " " + txt + " ")
END
SETCURSOR(1)
#else
WFORMAT(0,1,0,1)
IF txt!=NIL
    @ -1,1 SAY " " + txt + " "
END
#endif
RETURN aFeld


******************
PROC swclose(aFeld)
//Fenster schlieen
wselect(aFeld[2])
wclose()
wselect(aFeld[1])
#ifdef __XPP__
SETCURSOR(aFeld[3])
#endif
RETURN


************
FUNC futa(f, w)
//Zeile mit Funktionstastenbelegung
LOCAL x:=1, nSC:=SETCURSOR(), ws:=wselect(), aft:={}
IF w==NIL
    w := 8
END
wselect(0)
DISPBEGIN()
aft := {SAVESCREEN(AppMaxRow,0,AppMaxRow,AppMaxCol), nSC}
#ifdef __XPP__
SETCOLOR(farbe1)
#endif
@ AppMaxRow,0 SAY SPACE(AppMaxCol+1)
SETPOS(AppMaxRow,0)
WHILE x <= LEN(f)
    @ AppMaxRow, COL()+1 SAY PADR(f[x], w) COLOR farbe2
    x++
END
DISPEND()
wselect(ws)
#ifdef __XPP__
SETCOLOR(farbe1)
#endif
RETURN aft


****************
PROC restfuta(s)
//Zeile mit Funktionstastenbelegung wiederherstellen
LOCAL ws:=wselect()
wselect(0)
RESTSCREEN(AppMaxRow,0,AppMaxRow,AppMaxCol,s[1])
wselect(ws)
SETCURSOR(s[2])
RETURN


***********
FUNC upn(s)
//neue Infozeile ausgeben, alte zurckgeben
LOCAL nSC:=SETCURSOR(), ws:=wselect(), ret:=""
wselect(0)
ret:=ALLTRIM(CHARODD(SAVESCREEN(0,1,0,AppMaxCol-16)))
@ 0,1 SAY " " + PADR(IF(s=NIL, "", s), AppMaxCol-18) COLOR farbe2
wselect(ws)
SETCURSOR(nSC)
RETURN ret

