****************************************************************
*   xbasezusatz.prg (Modul von SIMAWI)
*   allgemeine Funktionen
*
*   SINUS Messtechnik GmbH, Steffen Schubert
*   Minimiert fr XSIMA (Textmodus), David Mackenzie
****************************************************************

#include "sima.ch"

//////////////////////////////////////////////////////////////////////
//
//  Following (to end of AX_SetScope) copied from ADSEXT.PRG - see also ACE.HLP
//  Copyright:
//   Alaska Software, (c) 1999-2006. All rights reserved.
//
//  Contents:
//       This sample shows how to use the ADS client API. See the READ.ME file.
//
//  Remarks:
//       The functions AX_SetScope() and AX_ClrScope() are wrapped. However,
//       a few more functions are implemented which are required for most of
//       the API functions.
//
//////////////////////////////////////////////////////////////////////

/*------------------- INCLUDES ----------------------------*/
#include "ads.ch"
#include "dll.ch"
#include "dmlb.ch"
#include "error.ch"
#include "adsdbe.ch"
#include "common.ch"
#include "Font.ch"
#include "Gra.ch"

/*------------------- DEFINES  ----------------------------*/

/* NOTE:
 * These defines must match the define constants from ACE.H !
 *
 * ACE.H cannot be included here because it's a C-file
 */
#define ACE_SUCCESS             0
#define ADS_RAWKEY              1
#define ADS_STRINGKEY           2
#define ADS_DOUBLEKEY           4
#define ADS_DYNAMIC_AOF         0
#define ADS_RESOLVE_IMMEDIATE   1
#define ADS_RESOLVE_DYNAMIC     2

#define AE_INVALID_HANDLE       5018
#define AE_INVALID_TABLE_HANDLE 5023
#define AE_NO_FILTER            5037
#define AE_NO_SCOPE             5038

/*
 * Name of the ADS client DLL
 */
#define ADSDLL      "ACE32.DLL"

/*------------------- MODULE GLOBAL DATA -------------------*/

/*
 * store handles and DLL calls in STATICs for high performance
 */
STATIC snAdsDllHandle    := 0
STATIC scDllGetLastError := ""

/*------------------- INIT/EXIT PROC'S  -------------------*/

INIT PROCEDURE ADSEXTI()
    snAdsDllHandle    := DllLoad( ADSDLL )
    scDllGetLastError := DllPrepareCall( snAdsDllHandle, DLL_STDCALL, "AdsGetLastError"   )
RETURN


EXIT PROCEDURE ADSEXTE()
    DllUnload( snAdsDllHandle )
RETURN


/*------------------- LOCAL FUNCTIONS   -------------------*/

/*
 * check rc , raise an error if necessary
 */
STATIC FUNCTION AdsExtErrorCheck( rc, nIgnoreError )
   LOCAL oErr
   LOCAL nLen    := 0
   LOCAL cBuffer := Space(128)
   LOCAL cWord   := W2Bin(Len(cBuffer))   /* 16 bit buffer */
   LOCAL rc_old  := rc

   IF nIgnoreError == NIL
      nIgnoreError := {}
   ENDIF

   IF rc != ACE_SUCCESS .AND. ASCAN(nIgnoreError, rc) == 0
      DllExecuteCall( scDllGetLastError, @rc, @cBuffer, @cWord )
      nLen               := Bin2W( cWord )
      cBuffer            := SubStr( cBuffer, 1, nLen )
      IF rc_old == AE_INVALID_HANDLE
         cBuffer := 'Check "Ausfhren in" Verzeichnis in Verknpfung!;' + cBuffer
      END
      oErr               := Error():new()
      oErr:genCode       := XPP_ERR_NOTABLE
      oErr:description   := cBuffer
      oErr:subCode       := rc
      oErr:canDefault    := .F.
      oErr:canRetry      := .T.
      oErr:canSubstitute := .T.
      oErr:operation     := ProcName(2) + "(" + LTrim(Str(ProcLine(2))) + ")"
      oErr:subSystem     := ADSDLL
      RETURN Eval( ErrorBlock(), oErr )
   ENDIF
RETURN rc

/*
 * get table handle of current workarea
 */
STATIC FUNCTION AdsGetTableHandle()
RETURN(DbInfo(ADSDBO_TABLE_HANDLE))

/*
 * get index handle of leading index of the current workarea
 */
STATIC FUNCTION AdsGetIndexHandle()
RETURN(OrdInfo(ADSORD_INDEX_HANDLE))


/*
 * AX_ClrScope()
 * clear top or bottom scope
 */
FUNCTION AX_ClrScope( nScopeType )
   LOCAL rc, hIndex

   IF nScopeType == NIL
      RETURN .F.
   ENDIF

   /*
    * retrieve current index handle
    */
   hIndex := AdsGetIndexHandle()
   /* call ADS API
    */
   rc := DllCall( snAdsDllHandle, DLL_STDCALL, "AdsClearScope", hIndex, nScopeType )
   AdsExtErrorCheck( rc )

RETURN rc == ACE_SUCCESS


/*
 * AX_SetScope( [<nScopeType>] [, <xValue>] ) -> xValue
 * set scope
 */
FUNCTION AX_SetScope( nScopeType, xScope )
   LOCAL rc, hIndex
   LOCAL cLen, nLen, nType, cType
   LOCAL cBuffer /* decouple passed value (ANSI/OEM conv!) */

   /*
    * retrieve current index handle
    */
   hIndex := AdsGetIndexHandle()

   /*
    * no scope value: query current scope value
    */
   IF PCount() == 1
      cBuffer := space(128)
      cLen := W2Bin(len(cBuffer))
      rc := DllCall( snAdsDllHandle, DLL_STDCALL, "AdsGetScope", hIndex, ;
                     nScopeType, @cBuffer, @cLen )
      AdsExtErrorCheck( rc, {AE_NO_SCOPE} )

      cBuffer := SubStr( cBuffer, 1, Bin2W(cLen))

      RETURN cBuffer
   ENDIF

   cType := Valtype( xScope )

   /*
    * check type of scope value and
    * convert to character
    */
   DO CASE
   CASE cType == "C"
      nLen   := Len( xScope )
      nType  := ADS_STRINGKEY

      /* convert to ANSI if we are running OEM
       */
      IF Set( _SET_CHARSET ) == CHARSET_OEM
         cBuffer := ConvToAnsiCp( xScope )
      ELSE
         cBuffer := xScope
      ENDIF

   CASE cType == "D"
      nLen    := 8
      nType   := ADS_STRINGKEY
      cBuffer := DTOS( xScope )

   CASE cType == "L"
      nLen    := 1
      nType   := ADS_STRINGKEY
      cBuffer := IIF( xScope, "T", "F" )

   CASE cType == "N"
      cBuffer    := f2bin( xScope )
      nLen       := LEN( cBuffer )
      nType      := ADS_DOUBLEKEY

   OTHERWISE
       RETURN NIL
   ENDCASE

   rc := DllCall( snAdsDllHandle, DLL_STDCALL, "AdsSetScope", hIndex, ;
                  nScopeType, @cBuffer, nLen, nType )
   AdsExtErrorCheck( rc )

RETURN xScope

////////////////////////////////////////////////////////////////////////////////
//  End of copied ADSEXT.PRG code
//  For the following, see redefinitions of "SET FILTER TO <x>" etc. in SIMA.CH
////////////////////////////////////////////////////////////////////////////////

/*
 * AX_SetFilter( [<cCondition>] ) -> NIL
 */
FUNCTION AX_SetFilter(cCondition)
LOCAL rc, hTable, nType, cBuffer // decouple passed value (ANSI/OEM conv!)
LOCAL i:=0

IF EMPTY(cCondition)
    RETURN AX_ClearFilter()
END

cCondition += CHR(0)
/* convert to ANSI if we are running OEM */
IF SET(_SET_CHARSET) == CHARSET_OEM
    cBuffer := ConvToAnsiCp(cCondition)
ELSE
    cBuffer := cCondition
ENDIF
nType := ADS_RESOLVE_DYNAMIC + ADS_DYNAMIC_AOF
hTable := AdsGetTableHandle()   // retrieve current table handle
rc := DllCall(snAdsDllHandle, DLL_STDCALL, "AdsSetAOF", hTable, ;
              @cBuffer, nType)
AdsExtErrorCheck(rc, {})

RETURN NIL

/*
 * AX_GetFilter() -> cFilter
 */
FUNCTION AX_GetFilter()
LOCAL rc, hTable, cLen, cFilter:=SPACE(999)

hTable := AdsGetTableHandle()   // retrieve current table handle

cLen := W2Bin(LEN(cFilter))
rc := DllCall(snAdsDllHandle, DLL_STDCALL, "AdsGetAOF", hTable, ;
              @cFilter, @cLen)
AdsExtErrorCheck(rc, {AE_INVALID_TABLE_HANDLE, AE_NO_FILTER})

cFilter := ConvToOemCP(SUBSTR(cFilter, 1, Bin2W(cLen)))

RETURN cFilter

/*
 * AX_ClearFilter() -> NIL
 */
FUNCTION AX_ClearFilter()
LOCAL rc, hTable

hTable := AdsGetTableHandle()   // retrieve current table handle

rc := DllCall(snAdsDllHandle, DLL_STDCALL, "AdsClearAOF", hTable)
AdsExtErrorCheck(rc, {AE_INVALID_TABLE_HANDLE, AE_NO_FILTER})

RETURN NIL

/* SCOPE redefinitions unnecessary?
FUNCTION OrdScope(nScope, expNewValue)
RETURN AX_SetScope(nScope, expNewValue)
*/

////////////////////////////////////////////////////////////////////////////////
//  End of ADS client API interface functions
////////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////
//
//  APPSYS.PRG
//
//  Copyright:
//      Alaska Software, (c) 1997-2009. Alle Rechte vorbehalten.
//
//  Inhalt:
//      AppSys() - Standard Applikationsfenster erzeugen
//
//  Bemerkungen:
//      Diese Datei ist Teil der XppRt0.lib.
//
//  Syntax:
//      Die Funktion AppSys() wird automatisch beim Programstart
//      aufgerufen.
//
//////////////////////////////////////////////////////////////////////

#include "xbp.ch"

****************************************************************************
* Funktion AppSys() um Standard-Ausgabegeraete zu erzeugen
****************************************************************************
PROCEDURE AppSys()
RETURN  // Von MyAppSys() ersetzt, aufgerufen nachdem Config-Datei gelesen

PROCEDURE MyAppSys()

  LOCAL nAppType := AppType()
  LOCAL aSizeDesktop, aPos, nFile:=0, nFontHeight:=0, nFontWidth:=0,;
        cBuffer:=SPACE(9999), nLen:=0, aFontBin:={}, i:=0, wi:=0, cDummy:=" "

  DO CASE

    // Anwendung wurde im PM Modus gelinkt, eine XbpCrt Instanz
    // ist zu erzeugen.
    CASE nAppType == APPTYPE_PM

      oFont := XbpFont():new()
      oFont:create("12.Alaska Crt")
      nFile := FOpen(cFontFile, FO_READ)
      IF nFile <> -1
        nLen := FRead(nFile, @cBuffer, LEN(cBuffer))
        FClose(nFile)
        aFontBin := Bin2Var(cBuffer)
        FOR i := 1 TO LEN(aFontBin)
            oFont:&(aFontBin[i,1]) := aFontBin[i,2]
        NEXT
      END
      oFont:configure()
      nFontHeight := oFont:height
      nFontWidth  := oFont:width

      // Bestimmen der Fensterposition (Anordnen in der Mitte
      // des Desktop-Fensters)
      aSizeDesktop := AppDesktop():currentSize()
      aPos         := { (aSizeDesktop[1]-((AppMaxCol+1) * nFontWidth))  /2, ;
                        (aSizeDesktop[2]-((AppMaxRow+1) * nFontHeight)) /2  }

      // XbpCRT-Fenster erzeugen
      oCrt := XbpCrt():New ( NIL, NIL, aPos, AppMaxRow+1, AppMaxCol+1 )
      oCrt:close := {|| Quit_App()}
//      oCrt:icon  := ICON_APP
      oCrt:Create()
      oCrt:SetFont(oFont)

      // Presentation Space initialisieren
      oCrt:PresSpace()

      // XbpCrt wird aktives Fenster und Ausgabegert
      SetAppWindow ( oCrt )

      // XbToolsIII Textmodus-Fenstersystem initialisieren
      WInit()
      WBoard()
      WMouseSelect(.f.)

      // Vllig sinnloser Vorgang nur um Problem zu vermeiden, dass bei
      // einer Textauswahl (kopieren) mit der Maus whrend eines READs
      // der ausgewhlten Bereich nicht hervorgehoben ist...
      wi := wopen(1,0,MAXROW()-1,MAXCOL())
      @ 0,0 GET cDummy
      KEYBOARD K_ENTER
      READ
      wclose(wi)


    // Anwendung wurde im VIO oder NOVIO Modus gelinkt, eine RootCrt
    // Instanz ist zu erzeugen.
    CASE nAppType == APPTYPE_VIO .OR. nAppType == APPTYPE_NOVIO

      // RootCrt Instanz erzeugen.
      //
      // Die IVar :CreateBuffer bestimmt wie sich das RootCrt verhaelt
      // wenn es in einem Shell Fenster erzeugt wird.
      //
      //   :CreateBuffer == .T. : Das RootCrt wird einen eigenen Bildschirm
      //                          Puffer erzeugen der dann fuer die Darstellung
      //                          verwendet wird. Dies hat zur Folge das
      //                          Ausgaben mit OutStd() nicht mit anderen
      //                          Text Ausgaben (z.B: Qout(), Dispbox() gemischt
      //                          werden koennen.
      //   :CreateBuffer == .F. : Es wird kein eigener Bildschirm Puffer erzeugt
      //                          und der des Shell Fensters verwendet. OutStd()
      //                          Ausgaben werden nun gleichzeitig dargestellt.
      oCrt := RootCrt():New()
      oCrt:CreateBuffer := .T.
      oCrt:Create()

      // RootCrt wird aktives Fenster und Ausgabegert.
      SetAppWindow ( oCrt )

      WINIT()

  ENDCASE

RETURN


// EOF (APPSYS.PRG)

PROCEDURE SelectFont()
LOCAL i:=0, nStartItem:=0, oFontLocal, aFontList:={}, aFontNames:={},;
      aFixedFonts:={}, nWahl:=0, nFile:=0, aFontBin:={},;
      aFontIVars := { "antiAliased", "baseLine", "bold", "codePage",;
                      "compoundName", "dbcs", "familyName", "fixed",;
                      "generic", "height", "italic", "kerning", "mbcs",;
                      "nominalPointSize", "outlined", "strikeout",;
                      "underscore", "vector", "weightClass", "width",;
                      "widthClass" }  // Instanzvariablennamen fr xbpFont

oFontLocal := XbpFont():new()
oFontLocal:fixed := .t.  // Sollte als Filter wirken, tut's aber nicht!
aFontList := oFontLocal:list()
FOR i := 1 TO LEN(aFontList)
    IF aFontList[i]:fixed .AND. aFontList[i]:nominalPointSize < 18
        AADD(aFontNames, STR(aFontList[i]:nominalPointSize, 2) + " Pt. " +;
                         "(Breite " + STR(aFontList[i]:width, 2) +;
                         ", Hhe " + STR(aFontList[i]:height, 2) + " Pixel, " +;
                         "Codepage " + STR(aFontList[i]:codePage, 3) + ")  " +;
                         aFontList[i]:compoundName)
        AADD(aFixedFonts, aFontList[i])
        IF nStartItem == 0 .AND.;
           oFont:compoundName     == aFontList[i]:compoundName .AND.;
           oFont:width            == aFontList[i]:width .AND.;
           oFont:height           == aFontList[i]:height .AND.;
           oFont:codePage         == aFontList[i]:codePage .AND.;
           oFont:nominalPointSize == aFontList[i]:nominalPointSize
            nStartItem := LEN(aFontNames)
        END
    END
NEXT
nWahl := ACHOICE(0, 0, MAXROW(), MAXCOL(), aFontNames, , , nStartItem)
IF nWahl > 0
    oFont := aFixedFonts[nWahl]
    nFile := FCreate(cFontFile)
    IF nFile = -1
        ALERT("DOS-Fehler " + ALLTRIM(STR(Ferror())) + " - " + cFontFile)
    ELSE
        FOR i := 1 TO LEN(aFontIVars)
            AADD(aFontBin, {aFontIVars[i], oFont:&(aFontIVars[i])})
        NEXT
        FWrite(nFile, Var2Bin(aFontBin))
        FClose(nFile)
    END
    oCrt:SetFont(oFont)
END
RETURN


/*==============================================================================
Ersatzfunktionen fr Clipper RDD _CDXDBE
===============================================================================*/
FUNCTION Ordsetrelation( xArea, bKey, cKey)
RETURN DC_SetRelation( xArea, bKey, cKey, .T.)

FUNCTION OrdKeyDel(xKey)
RETURN OrdKeyRemove(xKey)

FUNCTION OrdKeyGoto(nRec)
DbGoTop()
DbSkip(nRec)
RETURN NIL

FUNCTION DBOrderInfo(nTyp, x1, x2, x3)
// wird nur fr DBORDERINFO(DBOI_CONDITION) untersttzt (FOR-Bedingung als String)
LOCAL xRet

  DO CASE
    CASE nTyp == DBOI_CONDITION
      IF empty(x1)
        xRet := OrdFor()
      ELSE
        xRet := OrdFor(x1)
      ENDIF
  ENDCASE
RETURN xRet

FUNCTION DBRecordInfo(nTyp, x1, x2)
// wird nur fr DBRecordInfo(2) untersttzt: Ist Satz gesperrt?
LOCAL xRet

  x1 := IIf(Valtype(x1)=="N",x1,RecNo())
  DO CASE
    CASE nTyp == 2
      xRet := AScan(DbRLockList(), x1) > 0
  ENDCASE
RETURN xRet

/*==============================================================================
Redefinition of Clipper 5.3 compatible menu functions
===============================================================================*/

/*==============================================================================
Menu definition (simulates Clipper menu object functions)
===============================================================================*/
FUNCTION TopBar(x1, x2, x3, x4)
RETURN MenuArr():new()

FUNCTION PopUp(oTopBar)
RETURN MenuArr():new()

FUNCTION MenuItem(x1, x2, x3, x4, x5)
RETURN {STRTRAN(STRTRAN(x1,"&","~"),CHR(196),"@"),;
        IIF(x2==NIL, {}, x2),;
        IIF(x3==NIL, {}, x3),;
        IIF(x4==NIL, {}, x4),;
        IIF(x5==NIL, {}, x5)}

/*==============================================================================
MenuArr class - just collects entries via AddItem...
===============================================================================*/
Class MenuArr
   EXPORTED:
      VAR aItems
      VAR ColorSpec
      METHOD init, AddItem
ENDCLASS

METHOD MenuArr:init()
::aItems := {}
RETURN self

METHOD MenuArr:AddItem(aItem)
AADD(::aItems, aItem)
RETURN ::aItems

/*==============================================================================
Retrieval of entries in format required by DC_MENUMAIN()
===============================================================================*/
FUNCTION DC_MM_aMenus(oMenuBar)
LOCAL aMenus:={}
FOR i := 1 TO LEN(oMenuBar:aItems)
    AADD(aMenus, oMenuBar:aItems[i,1])
NEXT
RETURN {aMenus}

FUNCTION DC_MM_aSubMenus(oMenu, cMenu)
LOCAL i:=0, aSubItems:={}, aSubMenus:={}, aSubBlocks:={}, aSubMess:={}
FOR i := 1 TO LEN(oMenu:aItems)
    IF Valtype(oMenu:aItems[i,2]) == "O"
        aSubItems := DC_MM_aSubMenus(oMenu:aItems[i,2], oMenu:aItems[i,1])
        IF cMenu == NIL   // Top level is handled differently - see DC_MM_aMenus()
            AADD(aSubMenus,  aSubItems[2])
            AADD(aSubBlocks, aSubItems[3])
            AADD(aSubMess,   aSubItems[4])
        ELSE
            AADD(aSubMenus,  aSubItems)
            AADD(aSubBlocks, {})
            AADD(aSubMess,   oMenu:aItems[i,4])
        END
    ELSE
        AADD(aSubMenus,  oMenu:aItems[i,1])
        AADD(aSubBlocks, oMenu:aItems[i,2])
        AADD(aSubMess, oMenu:aItems[i,4])
    END
NEXT
IF cMenu == NIL
    RETURN {aSubMenus, aSubBlocks, aSubMess}
END
RETURN {cMenu, aSubMenus, aSubBlocks, aSubMess, {}}



/*==============================================================================
Redefinitions of MEMOLINE() and MLCOUNT() in sima.ch avoid known bugs
(see PDRs 3690 & 5531, respectively, in Alaska knowledge base).

#xtranslate MEMOLINE(<Params,...>) ;
       => MyMemoLine(<Params>)
#xtranslate MLCOUNT(<cString> [,<nLineLen>] [,<nTabSize>] [,<lWrap>]) ;
     =>  MyMemoLine(<cString>, <nLineLen>, -1, <nTabSize>, <lWrap>)

Thanks to Rajesh Kankaria (see PDR 3690)!
However, the following code is more Clipper-compatible & supports MLCOUNT()...
===============================================================================*/

func MyMemoLine( cString, nLineLen, nLine, nTabSize, lWrap )
local cLine,startrow,crlfchar,extract_len,checkstring

IF nLineLen == NIL
    nLineLen := 79
END
IF nLine == NIL
    nLine := 1
END
IF nTabSize == NIL
    nTabSize := 4
END
IF lWrap == NIL
    lWrap := .t.
END

startrow := 1
cLine    := ""

// Convert tab to space and soft CR (141) to CR
cString := strtran(cString, chr(9), space(nTabSize))
cString := strtran(cString, chr(141), chr(13))

while len(cString) > 0
    // Find position of rightmost space
    // The largest value is the maximum line length
    checkstring := substr(cString,1,nLineLen+1)
    crlfchar    := at(chr(13),checkstring)
    extract_len := 0
    if crlfchar > 0
        extract_len := crlfchar
    elseif lWrap .AND. len(cString) > nLineLen
        extract_len := rat(" ",checkstring)
    endif

    // No space or CR -- extract as much of line as possible
    if extract_len == 0
        extract_len := nLineLen
    endif

    // Extract the line
    cLine := left(checkstring,extract_len)
    if right(cLine,1) == chr(13)                // wrapped at CRLF
        cLine := left(cLine, len(cLine)-1)
    endif
    cLine := pad(cLine, nLineLen)

    // Required line extracted, exit
    if startrow == nLine
        exit
    endif

    startrow ++

    // Remove extracted part from string and continue with rest
    cString := substr(cString,extract_len+1)
    if left(cString,1) == chr(13)               // wrapped before CRLF
        cString := substr(cString, 3)
    elseif left(cString,1) == chr(10)           // wrapped at CRLF
        cString := substr(cString, 2)
    elseif left(cString,1) == " "               // wrapped without CRLF -
        cString := substr(cString, 2)           // ignore at most one space
    endif
    cLine := ""                                 // reinit it
end

return iif(nLine<0, startrow-1, cLine)


/*==============================================================================
                        Alaska PDR 2615 (s. sima.ch)
Symptom:
oGet:original always has the same value as oGet:buffer
Description:
The instance variable of the GET class :original has not the value of the
original variable as described in the documentation.
It gets the value of :buffer. So the undo() function won't work.
The workaround described in the former PDR 657 to use :getVar()
doesn't work too.
===============================================================================*/

CLASS GetX FROM Get
   EXPORTED:
   VAR original READONLY

   INLINE METHOD setFocus()
     ::get:setFocus()
     DEFAULT ::original TO ::varGet()
   RETURN( self )

   INLINE METHOD undo()
     IF ::hasFocus
       ::get:undo()
       Eval( ::block, ::original )
     ENDIF
   RETURN( self )

ENDCLASS


/*==============================================================================
Paste from clipboard with right mouse button
===============================================================================*/

PROCEDURE DoPaste()
LOCAL oCB, nChunkSize:=SET(_SET_TYPEAHEAD)-5

STATIC cText:=""

IF LEN(cText) == 0
    oCB := XbpClipBoard():new()
    IF oCB:open()
        cText := oCB:getBuffer(XBPCLPBRD_TEXT)
        IF EMPTY(cText)
            cText := ""   // sonst sturzt "IF LEN(cText) > 0" ab
        ELSE
            cText := MemoTrim(cText)
            IF RIGHT(cText,2) == cr // x0D0A
                cText := LEFT(cText, LEN(cText) - 2)
            END
        END
        oCB:close()
    END
    oCB:destroy()
END
IF LEN(cText) > 0
    IF LEN(cText) > nChunkSize
        KEYBOARD LEFT(cText, nChunkSize)
        PostAppEvent(xbeM_RbClick)
        cText := SUBSTR(cText, nChunkSize+1)
    ELSE
        KEYBOARD cText
        cText := ""
    END
END

RETURN


/*==============================================================================
Remove blank spaces before ends of lines
===============================================================================*/

FUNC MemoTrim(cText, cReplaceCR)
LOCAL nLen:=LEN(cText), nPos:=1, nAt:=0, cT:=""

IF cReplaceCR==NIL
    cReplaceCR := cr
END

WHILE nPos <= nLen
    nAt := AT(cr, cText, nPos)
    IF nAt == 0
        cT += RTRIM(SUBSTR(cText, nPos))
        nPos := nLen + 1
    ELSE
        cT += RTRIM(SUBSTR(cText, nPos, nAt-nPos)) + cReplaceCR
        nPos := nAt + LEN(cr)
    END
END

RETURN cT


#ifdef GUI
//////////////////////////////////////////////////////////////////////
//
//  Kopiert aus ALERTBOX.PRG
//
//  Copyright:
//      Alaska Software, (c) 1997-2006. Alle Rechte vorbehalten.
//
//  Inhalt:
//      Funktion AlertBox()
//
//      Die Funktion ist geeignet, die beiden Funktionen MsgBox() und
//      ConfirmBox() zu ersetzen. Sie arbeitet analog zu der Textmodus
//      Alert() Funktion, jedoch nur in GUI Anwendungen. Die Funktion
//      AlertBox() stellt ein modales Fenster dar.
//
//  Parameter:
//      <oOwner>   - Das Fenster auf dem die Alert Box zentriert wird.
//                   Der Standardwert ist SetAppWindow().
//
//      <cText>    - Message Text wie bei der Alert() Funktion.
//                   Das Semikolon ; wird als Zeilenumbruch verwendet
//
//      <aButton>  - Array mit Strings zur Anzeige der Pushbutton captions
//                   Der Standardwert ist { "Ok" }
//
//      <nSysIcon> - #define Konstante XBPSTATIC_SYSICON_* zur Anzeige
//                   eines Icons
//
//      <cTitle>   - Titel der Alert Box
//
//  Return:
//      Der Rckgabewert ist die numerische Ordinalposition des
//      gedrckten Pushbuttons oder Null, wenn keine Auswahl erfolgte.
//      (ESC Taste oder Fenster wurde geschlossen)
//
//  Bemerkungen:
//      Wenn kein Icon angezeigt wird, erfolgt die Anzeige von Text
//      zentriert im Fenster. Ansonsten wird der Text links ausgerichtet
//      neben dem Icon angezeigt. Die Pushbuttons werden unterhalb des
//      Textes zentriert.
//
//////////////////////////////////////////////////////////////////////

/*
 * Maximale Anzahl von Pushbuttons in der Alert Box
 */
#define  MAX_BUTTONS         4

#define  TEXT_COLOR          GRA_CLR_BLACK


/*
 * Diese Funktion arbeitet wie die Textmodus Alert() Funktion, jedoch nur
 * fr GUI Applikationen. Die Box wird auf dem angegebenen Fenster zentriert.
 */

FUNCTION AlertBox( oOwner, cText, aCaption, nSysIcon, cTitle )
   LOCAL nEvent   , mp1 , mp2, oXbp , lExit
   LOCAL oParent  , oDlg, oPS, oFocus, oDesktop
   LOCAL aPushBtn , aPbSize, nPbWidth, nPbDist, nSelect
   LOCAL oIcon    , aIconSize
   LOCAL aTextSize, aText
   LOCAL bSelect  , bKeyboard
   LOCAL i, imax  , nPos   , aPos    , aPos1  , aSize  , nXsize, nYsize

   DEFAULT oOwner   TO oCrt, ;
           cText    TO ""            , ;
           aCaption TO { "~OK" }     , ;
           cTitle   TO " "

   oDeskTop         := AppDesktop()


  /*
   * Dialogfenster der Alert Box erzeugen
   */
   oDlg             := XbpDialog():new( oDesktop, oOwner, , { 100, 100 }, , .F. )
//   oDlg:alwaysOnTop := .T.
   oDlg:minButton   := .F.
   oDlg:maxButton   := .F.
   oDlg:title       := cTitle
   oDlg:border      := XBPDLG_DLGBORDER
   oDlg:close       := {|mp1,mp2,obj| obj:hide(), lExit := .T. }
   oDlg:create()
   oDlg:drawingArea:SetFontCompoundName( oFont:compoundName )
   oDlg:drawingArea:paint := {|| Repaint( aText ) }


  /*
   * Icon erzeugen, falls erforderlich
   */
   IF nSysIcon <> NIL
      oIcon          := XbpStatic():new( oDlg:drawingArea, , {0,0}, {30,30} )
      oIcon:caption  := nSysIcon
      oIcon:type     := XBPSTATIC_TYPE_SYSICON
      oIcon:create()
      aIconSize      := oIcon:currentSize()
   ELSE
      aIconSize      := {0,0}
   ENDIF


  /*
   * Presentation Space und Font erzeugen, um die Gre der Pushbuttons
   * anhand der Font Metrics zu berechnen
   */
   oPS    := XbpPresSpace():new():create( oDlg:drawingArea:winDevice() )
   oPS:setFont( oFont )


  /*
   * Lngsten Caption String finden
   */
   imax   := Min( MAX_BUTTONS, Len( aCaption ) )
   nPos   := 0
   nXSize := 0

   FOR i:=1 TO imax
      IF Len( aCaption[i] ) > nXsize
         nXsize := Len( aCaption[i] )
         nPos   := i
      ENDIF
   NEXT


  /*
   * Gre der Pushbuttons berechnen
   */
   aPbSize    := GraQueryTextBox( oPS, aCaption[nPos] )
   aPbSize    := { aPbSize[3,1] - aPbSize[2,1] ;
                 , aPbSize[3,2] - aPbSize[2,2] }
   aPbSize[1] *= 1.5
   aPbSize[2] *= 2
   nPbDist    := aPbSize[1] / nXsize
   nPbWidth   := imax * ( aPbSize[1] + nPbDist )


  /*
   * Text zur Anzeige vorbereiten und Gre berechnen
   */
   oPS:setFont( oFont )
   aTextSize := PrepareText( oPS, @aText, cText )


  /*
   * Gre des Rahmens der Alert Box berechnen (XbpDialog Fenster)
   */
   nXsize := Max( nPbWidth + 20, aTextSize[1] + 20 ) + aIconSize[1] * 1.5
   nYsize := aTextSize[2] + 2 * aPbSize[2] + 10
   aSize  := oDlg:calcFrameRect( { 0, 0, nXsize, nYsize } )

   oDlg:setSize( { aSize[3], aSize[4] } )
   MoveToOwner( oDlg )
   aSize := oDlg:drawingArea:currentSize()


   IF nSysIcon == NIL
     /*
      * Kein Icon - Text im Fenster zentrieren
      */
      aPos := CenterPos( aTextSize, aSize )
   ELSE
     /*
      * Mit Icon - Text am Icon ausrichten
      */
      aPos    := { 0, 0 }
      aPos[1] := 1.75 * aIconSize[1]
   ENDIF
   aPos[2]    := 2.25 * aPbSize[2]
   AdjustPos( aPos, aTextSize, aText, nSysIcon )


  /*
   * Icon oben links am Text positionieren
   */
   IF oIcon <> NIL
      aPos[1] := aIconSize[1] / 3
      aPos[2] := aSize[2] - 1.5 * aIconSize[2]
      oIcon:setPos( aPos )
   ENDIF


   IF nSysIcon == NIL
     /*
      * Kein Icon - Pushbuttons im Fenster zentrieren
      */
      aPos    := CenterPos( {nPbWidth, 2 * aPbSize[2]}, aSize )
      aPos[1] += ( nPbDist / 2 )

   ELSEIF nPbWidth < aTextSize[1]
     /*
      * Die Breite aller Pushbuttons ist kleiner als die Textbreite.
      * Pushbuttons unterhalb vom Text zentrieren.
      */
      aPos    := CenterPos( {nPbWidth, 2 * aPbSize[2]}, aTextSize )
      aPos[1] += ( 1.5 * aIconSize[1] + ( nPbDist / 2 ) )

   ELSE
     /*
      * Pushbuttons am Text ausrichten
      */
      aPos[1] := 1.75 * aIconSize[1]
   ENDIF


   aPushBtn   := Array( imax )
   aPos[2]    := aPbSize[2] / 2
   nSelect    := 0
   bSelect    := {|mp1 ,mp2,obj| nSelect := obj:cargo, lExit := .T. }
   bKeyBoard  := {|nKey,mp2,obj| KeyHandler( nKey, obj, aPushBtn ) }


  /*
   * Pushbuttons erzeugen
   */
   FOR i:=1 TO imax
      aPushBtn[i] := XbpPushButton():new( oDlg:drawingArea,, aPos, aPbSize )
      aPushBtn[i]:caption  := aCaption[i]
      aPushBtn[i]:cargo    := i
      aPushBtn[i]:tabstop  := .T.
      aPushBtn[i]:activate := bSelect
      aPushBtn[i]:keyboard := bKeyboard

      IF imax > 1
         IF i == 1
            aPushBtn[i]:group := XBP_BEGIN_GROUP
         ELSEIF i == imax
            aPushBtn[i]:group := XBP_END_GROUP
         ELSE
            aPushBtn[i]:group := XBP_WITHIN_GROUP
         ENDIF
      ENDIF

      aPushBtn[i]:create()
      aPos[1] += ( aPbSize[1] + nPbDist )
   NEXT


   oDlg:setModalState( XBP_DISP_APPMODAL )
   oDlg:show()
   oFocus := SetAppFocus( aPushBtn[1] )

   lExit := .F.
   DO WHILE ! lExit
      nEvent := AppEvent( @mp1, @mp2, @oXbp )
      oXbp:handleEvent( nEvent, mp1, mp2 )
   ENDDO

  /*
   * Aufrumen: Modalitt zurcksetzen, grafisches Segment freigeben,
   *            Dialog freigeben und Fokus setzen.
   */
   oDlg:setModalState( XBP_DISP_MODELESS )
   GraSegDestroy( oPS, aText[2] )
   oDlg:destroy()
   SetAppFocus( oFocus )

RETURN nSelect



/*
 * Tastenverarbeitung fr Pushbuttons in der Alert Box.
 * Das Array, das die Pushbuttons enthlt wird hier nicht gebraucht.
 * (knnte aber von jemand anderem gebraucht werden...)
 */
STATIC PROCEDURE KeyHandler( nKey, oButton, aPushButtons )

   DO CASE
   CASE nKey == xbeK_ESC
      PostAppEvent( xbeP_Close,,, oButton:setParent():setParent() )
   CASE nKey == xbeK_RETURN
      PostAppEvent( xbeP_Activate,,, oButton )
   ENDCASE

RETURN



/*
 * Gre der Flche fr den Anzeigetext berechnen
 */
STATIC FUNCTION PrepareText( oPS, aText, cText )
   LOCAL aAttr, aTextBox, nMaxWidth, nMaxHeight, cLine, aLine

   aText         := {}
   nMaxWidth     := 0
   nMaxHeight    := 0
   aAttr         := Array( GRA_AS_COUNT )

   aAttr[ GRA_AS_COLOR ] := TEXT_COLOR
   oPS:setAttrString( aAttr )

   DO WHILE ! Empty( cText )
      cLine      := CutStr( ";", @cText )
      IF Empty( cLine )
         cLine   := " "
      ENDIF
      aLine      := { {0,0}, cLine, 0, 0 }
      aTextBox   := GraQueryTextBox( oPS, aLine[2] )
      aLine[3]   := aTextBox[3,1] - aTextBox[2,1]  // Breite des Substrings
      aLine[4]   := aTextBox[3,2] - aTextBox[2,2]  // Hhe des Substrings
      nMaxWidth  := Max( nMaxWidth, aLine[3] )
      nMaxHeight += ( 2 + aLine[4] )
      AAdd( aText, aLine )
   ENDDO

   aText         := { oPS, 0, aText, aAttr }

RETURN { nMaxWidth, nMaxheight }



/*
 * GraStringAt() Positionen fr zentrierte oder links ausgerichtete
 * Anzeige korrigieren
 */
STATIC PROCEDURE AdjustPos( aPos, aSize, aText, nSysIcon )
   LOCAL aLines  := aText[3]
   LOCAL i, imax := Len( aLines )

   IF nSysIcon == NIL
     /*
      * Kein Icon - Text zentriert anzeigen
      */
      FOR i := 1 TO imax
         aLines[i,1,1] := aPos[1] + ( (aSize[1] - aLines[i,3]) / 2 )
         aLines[i,1,2] := aPos[2] + aSize[2] - i * ( 2 + aLines[i,4] )
      NEXT
   ELSE
     /*
      * Mit Icon - Text links ausgerichtet anzeigen
      */
      FOR i := 1 TO imax
         aLines[i,1,1] := aPos[1]
         aLines[i,1,2] := aPos[2] + aSize[2] - i * ( 2 + aLines[i,4] )
      NEXT
   ENDIF
RETURN



/*
 * Text mittels grafischem Segment neu zeichnen
 */
STATIC PROCEDURE Repaint( aText )
   LOCAL oPS    := aText[1]
   LOCAL nID    := aText[2]
   LOCAL aLines := aText[3]
   LOCAL aAttr  := aText[4]

   oPS:setAttrString( aAttr )

   IF nID == 0
      nID := GraSegOpen( oPS )
      AEval( aLines, {|a| GraStringAt( oPS, a[1], a[2] ) } )
      GraSegClose( oPS )
      aText[2] := nID
   ENDIF

   GraSegDraw( oPS, nID )

RETURN



/*
 * Fenster innerhalb Parent-Window verschieben wobei der Bezugspunkt fr
 * die Koordinaten das Owner-Windows ist. Standardmig wird ein
 * Fenster auf seinem Owner zentriert.
 */
STATIC PROCEDURE MoveToOwner( oDlg, aPos )
   LOCAL oOwner:=oCrt
   LOCAL aPos1:=oOwner:currentPos()

   DEFAULT aPos TO CenterPos( oDlg:currentSize(), oOwner:currentSize() )

   aPos[1] += aPos1[1]
   aPos[2] += aPos1[2]

   oDlg:setPos( aPos )
RETURN



/*
 * Zentrierte Position fr ein Fenster aus der Fenstergre und einer
 * Referenzgre berechnen
 */
STATIC FUNCTION CenterPos( aSize, aRefSize )
RETURN { Int( (aRefSize[1] - aSize[1]) / 2 ) ;
       , Int( (aRefSize[2] - aSize[2]) / 2 ) }



/*
 * String an einem Delimiter trennen und Delimiter entfernen
 */
STATIC FUNCTION CutStr( cCut, cString )
   LOCAL cLeftPart, i := At( cCut, cString )

   IF i > 0
      cLeftPart := Left( cString, i-1 )
      cString   := SubStr( cString, i+Len(cCut) )
   ELSE
      cLeftPart := cString
      cString   := ""
   ENDIF

RETURN cLeftPart

//////////////////////////////////////////////////////////////////////
//
//  Ende ALERTBOX.PRG
//
//////////////////////////////////////////////////////////////////////
#endif
